"""
JSON Loading utilities
"""
from json import loads
import delgado
from delgado.exceptions import Forbidden, InvalidFormat


def loader(string, allowed=None):
    allowed = allowed or delgado.config.get('allowed', [])
    try:
        obj = loads(string)
        for exe in obj.keys():
            if exe not in allowed:
                raise Forbidden('Executable %s, is not allowed' % exe)

        return format_command(obj)
    except ValueError:
        msg = 'unable to parse unexpected input: %s' % repr(string)
        raise InvalidFormat(msg)


def format_command(obj):
    try:
        if not obj:
            raise AttributeError
        executable = list(obj)[0]
        arguments = obj[executable]
        return [executable] + arguments
    except AttributeError:
        msg = "JSON format should be like {'executable': [arguments]}" \
              " but received: %s" % repr(obj)
        raise InvalidFormat(msg)



import socket
import os
from subprocess import call
from tambo import Transport
import delgado
from delgado.loader import loader
from delgado.exceptions import Forbidden, InvalidFormat, Reconnect
from delgado import logger


class Server(object):

    _help = """
Run the server that listens on a unix socket.

--allowed           The executable allowed to run for this listener
    """

    def __init__(self, argv, connection=None):
        self.argv = argv
        self.connection = connection

    def parse_args(self):
        options = ['--allowed']
        parser = Transport(self.argv, options=options)
        parser.catch_help = self._help
        parser.parse_args()
        delgado.config['allowed'] = parser.get('--allowed') or []
        engine = Engine(connection=self.connection)
        engine.run_forever()


class Engine(object):

    def __init__(self, socket_location=None, connection=None):
        self.socket_location = socket_location or '/tmp/delgado.sock'
        if connection:
            self.connection = connection  # should be a callable
        else:
            self.connection = self.make_connection

    def run_forever(self):
        while True:
            try:
                logger.debug('creating a new connection')
                self.run()
            except Reconnect:
                pass

    def make_connection(self):
        s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        try:
            os.remove(self.socket_location)
        except OSError:
            pass
        s.bind(self.socket_location)
        s.listen(1)
        conn, addr = s.accept()
        logger.debug('connection ready and listenting')
        return conn

    def run(self):
        conn = self.connection()
        raw_data = conn.recv(1024)
        try:
            if not raw_data:
                raise Reconnect
            data = loader(raw_data)
            logger.info("Running command: %s" % data)
            call(data)
        except (InvalidFormat, Forbidden) as error:
            logger.error(str(error))
            raise Reconnect
        finally:
            conn.close()
            logger.debug('connection closed')



import pkg_resources
import sys

from tambo import Transport
from delgado import logger
import delgado
from delgado.server import Server
from delgado.decorators import catches


class Delgado(object):
    _help = """
delgado: A utility to run in the foreground and listen for commands
to execute from the network to the terminal
descriptive tests.
Version: %s

Global Options:
--log, --logging    Set the level of logging. Acceptable values:
                    debug, warning, error, critical

run                 Run the server, listening on a unix socket.


Plugins:
%s

    """

    mapper = {'run': Server}

    def __init__(self, argv=None, parse=True):
        self.plugin_help = "No plugins found/loaded"
        if argv is None:
            argv = sys.argv
        if parse:
            self.main(argv)

    def help(self):
        return self._help % (delgado.__version__, self.plugin_help)

    def enable_plugins(self):
        """
        Load all plugins available, add them to the mapper and extend the help
        string with the information from each one
        """
        plugins = _load_library_extensions()
        for plugin in plugins:
            self.mapper[plugin._delgado_name_] = plugin
        self.plugin_help = ''.join(['%-19s %s' % (
            plugin._delgado_name_, getattr(plugin, 'help_menu', ''))
            for plugin in plugins])

    @catches(KeyboardInterrupt)
    def main(self, argv):
        options = [['--log', '--logging']]
        parser = Transport(argv, mapper=self.mapper,
                           options=options, check_help=False,
                           check_version=False)
        parser.parse_args()
        delgado.config['verbosity'] = parser.get('--log', 'info')
        self.enable_plugins()
        parser.catch_help = self.help()
        parser.catch_version = delgado.__version__
        parser.mapper = self.mapper
        if len(argv) <= 1:
            return parser.print_help()
        parser.dispatch()
        parser.catches_help()
        parser.catches_version()


def _load_library_extensions():
    """
    Locate all setuptools entry points by the name 'delgado_handlers'
    and initialize them.
    Any third-party library may register an entry point by adding the
    following to their setup.py::

        entry_points = {
            'delgado_handlers': [
                'plugin_name = mylib.mymodule:Handler_Class',
            ],
        },

    `plugin_name` will be used to load it as a sub command.
    """
    group = 'delgado_handlers'
    entry_points = pkg_resources.iter_entry_points(group=group)
    plugins = []
    for ep in entry_points:
        try:
            logger.debug('loading %s' % ep.name)
            plugin = ep.load()
            plugin._delgado_name_ = ep.name
            plugins.append(plugin)
        except Exception as error:
            logger.error("Error initializing plugin %s: %s" % (ep, error))
    return plugins






from pytest import raises
from delgado import main


class TestMain(object):

    def test_get_help_with_no_args(self, capsys):
        with raises(SystemExit):
            main.Delgado([])
        out, err = capsys.readouterr()
        assert err == ''
        assert 'Version:' in out

    def test_load_pytest(self, capsys):
        main.Delgado(['delgado', '--log', 'debug'])
        out, err = capsys.readouterr()
        assert err == ''
        assert 'loading pytest' in out

    def test_info_logger_doesnt_show_plugin_loading(self, capsys):
        main.Delgado(['delgado', '--logging', 'info'])
        out, err = capsys.readouterr()
        assert 'loading pytest' not in out

    def test_unkown_command(self, capsys):
        main.Delgado(['delgado', '--foo', 'info'])
        out, err = capsys.readouterr()
        assert 'Unknown command' in out



from tambo import Transport
import delgado
from delgado.server import Engine


class Pytest(object):

    help_menu = 'A handler for running py.test commands'
    _help = """
Run a base socket listener that allows py.test commands.

--socket-location   The location for the socket (defaults
                    to /tmp/pytest.sock)
    """

    def __init__(self, argv):
        self.argv = argv

    def parse_args(self):
        parser = Transport(self.argv, options=['--socket-location'])
        parser.catch_help = self._help
        parser.parse_args()
        location = parser.get('--socket-location') or '/tmp/pytest.sock'
        delgado.config['allowed'] = ['py.test']
        engine = Engine(socket_location=location)
        engine.run_forever()



from pytest import raises
from delgado import loader
from delgado.exceptions import InvalidFormat, Forbidden


class TestFormatCommand(object):

    def test_get_executable_first(self):
        obj = {'executable': [1, 2, 3]}
        result = loader.format_command(obj)
        assert result == ['executable', 1, 2, 3]

    def test_no_arguments_are_ok(self):
        obj = {'executable': []}
        result = loader.format_command(obj)
        assert result == ['executable']

    def test_raise_invalid_format(self):
        with raises(InvalidFormat):
            loader.format_command([])

    def test_get_correct_rpr(self):
        with raises(InvalidFormat) as exc:
            loader.format_command([])
        error = exc.value.args[0]
        assert 'received: []' in error

    def test_no_obj(self):
        with raises(InvalidFormat):
            loader.format_command({})


class TestLoader(object):

    def test_loader_simplest_obj(self):
        result = loader.loader("""{"foo": []}""", allowed=['foo'])
        assert result == ['foo']

    def test_loader_not_allowed(self):
        with raises(Forbidden):
            loader.loader("""{"foo": []}""")

    def test_unable_to_parse(self):
        with raises(InvalidFormat):
            loader.loader("""foo": []}""", allowed=['foo'])



from mock import Mock
from pytest import raises
from delgado import server


class TestServer(object):

    def setup(self):
        self.conn = Mock()
        self.conn.return_value = self.conn

    def set_debug_level(self):
        import delgado
        delgado.config['verbosity'] = 'debug'

    def test_run(self):
        # meh just excercising code really
        fake_conn = Mock(side_effect=TypeError)
        srv = server.Server([], connection=fake_conn)
        with raises(TypeError):
            srv.parse_args()

    def test_not_allowed(self, capsys):
        self.set_debug_level()
        self.conn.recv = Mock(side_effect=["""{"ls": []}""", TypeError])
        srv = server.Server([], connection=self.conn)
        with raises(TypeError):
            srv.parse_args()
        out, err = capsys.readouterr()
        assert 'ls, is not allow' in out
